﻿--[[---------------------------------------------------------------------------

Copyright (c) 2008-2017 by K. Scott Piel 
All Rights Reserved

E-mail: < kscottpiel@gmail.com >

This file is part of nUI.

The author no longer develops or directly supports this addon and has released it
into the public domain under the Creative Commons Attribution-NonCommercial 3.0 
Unported license - a copy of which is included in this distribution. This source
may not be distributed without said license.
	   
nUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
PURPOSE.  See the enclosed license for more details.
	
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS  "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT  LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.

--]]---------------------------------------------------------------------------

if not nUI_Profile then nUI_Profile = {}; end;

nUI_Profile.nUI_AzeriteBar = {};

local ProfileCounter = nUI_Profile.nUI_AzeriteBar;

-------------------------------------------------------------------------------

local background    = CreateFrame( "Frame", "nUI_AzeriteBarBackground", nUI_Dashboard.Anchor );
local frame         = nUI_Bars:createStatusBar( "nUI_AzeriteBar", nUI_Dashboard.Anchor );
frame.text          = frame:CreateFontString( "$parentLabel", "OVERLAY" );
frame.Super         = {};

frame.updateBar( nil, { 0, 0, 0, 0 } );
frame:EnableMouse( true );
background:SetAllPoints( frame );

-------------------------------------------------------------------------------
local function UpdateAzerite()

    local hasActiveAzeriteItem = C_AzeriteItem.HasActiveAzeriteItem()
    local azeriteItemLocation = C_AzeriteItem.FindActiveAzeriteItem();

    frame:EnableMouse( true );    

    frame:SetScript( "OnLeave", 
        function() 
            GameTooltip:Hide(); 
        end 
    );

    if not hasActiveAzeriteItem or not azeriteItemLocation then        
        frame:SetScript( "OnEnter", 
            function()                
                if not hasActiveAzeriteItem or not azeriteItemLocation then
                    frame:SetScript("OnEnter", function(self)
                        GameTooltip:SetOwner( frame );
                        GameTooltip:ClearLines()
                        GameTooltip:SetText(ERR_MUST_EQUIP_ARTIFACT,1,0.83,0)
                        GameTooltip:Show()
                    end)
                end
            end
        );
    end
    
    if not hasActiveAzeriteItem or not azeriteItemLocation then
        return
    end
    
    local xp, totalXP = C_AzeriteItem.GetAzeriteItemXPInfo(azeriteItemLocation);
    local currentLevel = C_AzeriteItem.GetPowerLevel(azeriteItemLocation);
    local xpToNextLevel = totalXP - xp;
    local pct = xp / totalXP

    frame:SetScript( "OnEnter", 
        function()
            local azeriteItemLocation = C_AzeriteItem.FindActiveAzeriteItem()
            local azeriteItem = azeriteItemLocation and Item:CreateFromItemLocation(azeriteItemLocation) or nil
            local azeriteItemName = azeriteItem and azeriteItem:GetItemName() or nil

            local neckItemLocation = ItemLocation:CreateFromEquipmentSlot(Enum.InventoryType.IndexNeckType)
            local headItemLocation = ItemLocation:CreateFromEquipmentSlot(Enum.InventoryType.IndexHeadType)
            local shoulderItemLocation = ItemLocation:CreateFromEquipmentSlot(Enum.InventoryType.IndexShoulderType)
            local chestItemLocation = ItemLocation:CreateFromEquipmentSlot(Enum.InventoryType.IndexChestType)

            local hasNeckItem = C_Item.DoesItemExist(neckItemLocation)
            local hasHeadItem = C_Item.DoesItemExist(headItemLocation)
            local hasShoulderItem = C_Item.DoesItemExist(shoulderItemLocation)
            local hasChestItem = C_Item.DoesItemExist(chestItemLocation)
            
            local headIsAzeriteEmpowered = hasHeadItem and C_AzeriteEmpoweredItem.IsAzeriteEmpoweredItem(headItemLocation)
            local shoulderIsAzeriteEmpowered = hasShoulderItem and C_AzeriteEmpoweredItem.IsAzeriteEmpoweredItem(shoulderItemLocation)
            local chestIsAzeriteEmpowered = hasChestItem and C_AzeriteEmpoweredItem.IsAzeriteEmpoweredItem(chestItemLocation)

            local headItem = headItemLocation and headIsAzeriteEmpowered and Item:CreateFromItemLocation(headItemLocation) or nil
            local shoulderItem = shoulderItemLocation and shoulderIsAzeriteEmpowered and Item:CreateFromItemLocation(shoulderItemLocation) or nil
            local chestItem = chestItemLocation and chestIsAzeriteEmpowered and Item:CreateFromItemLocation(chestItemLocation) or nil

            local headItemName = headItem and headItem:GetItemName() or ""
            local shoulderItemName = shoulderItem and shoulderItem:GetItemName() or ""
            local chestItemName = chestItem and chestItem:GetItemName() or ""

            local azeritePowerAvailable = " (" .. AZERITE_ITEM_LEVELED_UP_TOAST_UNLOCKED_MULTIPLE .. ")" 

            local headHasPowers = headItem and C_AzeriteEmpoweredItem.HasAnyUnselectedPowers(headItemLocation) and azeritePowerAvailable or ""
            local shoulderHasPowers = shoulderItem and C_AzeriteEmpoweredItem.HasAnyUnselectedPowers(shoulderItemLocation) and azeritePowerAvailable or ""
            local chestHasPowers = chestItem and C_AzeriteEmpoweredItem.HasAnyUnselectedPowers(chestItemLocation) and azeritePowerAvailable or ""

            local isEquippedAzeriteItem = hasNeckItem and (C_Item.GetItemInventoryType(azeriteItemLocation) == Enum.InventoryType.IndexNeckType)
            local mustEquip = (not hasNeckItem or not isEquippedAzeriteItem) and ERR_MUST_EQUIP_ARTIFACT or ""
            
            GameTooltip:SetOwner( frame );
            GameTooltip:ClearLines()
            GameTooltip:SetText(AZERITE_POWER_TOOLTIP_TITLE:format(frame.level, frame.xpToNextLevel), HIGHLIGHT_FONT_COLOR:GetRGB());
            GameTooltip:AddLine(AZERITE_POWER_TOOLTIP_BODY:format(azeriteItemName));
            GameTooltip:AddLine(mustEquip,HIGHLIGHT_FONT_COLOR:GetRGB())
            GameTooltip:AddLine(headItemName .. headHasPowers ,HIGHLIGHT_FONT_COLOR:GetRGB())
            GameTooltip:AddLine(shoulderItemName .. shoulderHasPowers ,HIGHLIGHT_FONT_COLOR:GetRGB())
            GameTooltip:AddLine(chestItemName .. chestHasPowers ,HIGHLIGHT_FONT_COLOR:GetRGB())

            GameTooltip:Show();
        end
    );

	if frame.azeriteItemName ~= azeriteItemName or frame.level ~= currentLevel or frame.xp ~= xp or frame.totalXP ~= totalXP or frame.xpToNextLevel ~= xpToNextLevel then
				
			frame.azeriteItemName  = azeriteItemName;
			frame.level = currentLevel;
			frame.xp      = xp;
			frame.totalXP      = totalXP;
			frame.xpToNextLevel      = xpToNextLevel;
            
			if frame.enabled and frame.bar.enabled then
				frame.updateBar( pct, frame.options.bar.color );
			end
			
			local label = AZERITE_POWER_TOOLTIP_TITLE:format(frame.level, frame.xpToNextLevel) 
			
			if frame.text.enabled and frame.text.value ~= label then				
				frame.text.value = label;
				frame.text:SetText( label );
                frame.text:SetAlpha(0.5)
				
			elseif not frame.text.enabled and frame.text.value then				
				frame.text.value = nil;
				frame.text:SetText( "" );
				
			end
		end		

end

-------------------------------------------------------------------------------
-- Allow external function to trigger update
function frame.setActive()
    frame.azeriteItemName = nil
    UpdateAzerite()
end

-------------------------------------------------------------------------------
-- Allow external function to identify whether this bar should be tracked
function frame.isTrackable()
    local hasActiveAzeriteItem = C_AzeriteItem.HasActiveAzeriteItem()
    local azeriteItemLocation = C_AzeriteItem.FindActiveAzeriteItem();
    local associated = frame.barID and frame.barID ~= 0
    return hasActiveAzeriteItem and azeriteItemLocation and not associated
end

-------------------------------------------------------------------------------
-- Event function
local function onAzeriteEvent( who, event, arg1, arg2 )
	
--	nUI_ProfileStart( ProfileCounter, "onFactionEvent", event );
	
	if event == "ADDON_LOADED" then
		
		if arg1 == "nUI" then		
			nUI:registerScalableFrame( frame );
			nUI:registerSkinnedFrame( frame );
		end
		
	elseif event == "PLAYER_ENTERING_WORLD" then
        if not frame.barID or frame.barID == 0 then
            frame:Hide()
        else
            UpdateAzerite();
        end

    -- Check for Azerite Item Equipment change
    elseif event == "PLAYER_EQUIPMENT_CHANGED" then
        UpdateAzerite()
    
    -- Check for Azerite Item Experience Change
	elseif event == "AZERITE_ITEM_EXPERIENCE_CHANGED" then 
        UpdateAzerite()
                
    -- Check for Azerite Item Power Level Change
    elseif event == "AZERITE_ITEM_POWER_LEVEL_CHANGED" then
        UpdateAzerite()
		
	end
	
--	nUI_ProfileStop();
	
end

background:SetScript( "OnEvent", onAzeriteEvent );
background:RegisterEvent( "ADDON_LOADED" );
background:RegisterEvent( "PLAYER_EQUIPMENT_CHANGED")
background:RegisterEvent( "PLAYER_ENTERING_WORLD" );
background:RegisterEvent( "AZERITE_ITEM_POWER_LEVEL_CHANGED")
background:RegisterEvent( "AZERITE_ITEM_EXPERIENCE_CHANGED" );

-------------------------------------------------------------------------------

frame.applyScale = function( scale )
	
--	nUI_ProfileStart( ProfileCounter, "applyScale" );
	
	local anchor  = scale and frame.anchor or nil;
	local scale   = scale or frame.scale or 1;
	local options = frame.options;
	
	frame.scale = scale;
	
	if options then
		
		local width       = options.width * scale * nUI.hScale;
		local height      = options.height * scale * nUI.vScale;
		local text        = frame.text;
		local font_size   = (options.label and options.label.fontsize or 12) * scale * 1.75 * nUI.vScale;
		local justifyH    = options.label  and options.label.justifyH or "CENTER";
		local justifyV    = options.label  and options.label.justifyV or "MIDDLE";
	
		-- set the bar size
		
		if frame.width  ~= width
		or frame.height ~= height
		then
			
			frame.width  = width;
			frame.height = height;
			
			frame:SetHeight( height );
			frame:SetWidth( width );

			frame.onSizeChanged();
			
		end
		
		-- set the text font size
					
		if text.font_size ~= font_size
		then
			
			-- first time here?
			
			if not text.font_size then 
				text.active  = true;
			end
	
			text.font_size = font_size;
			text:SetFont( nUI_L["font1"], font_size, "OUTLINE" );

		end
		
		-- show or hide the text based on whether or not there is a config for it
		
		text.enabled = options.label and options.label.enabled or false;
		
		if not text.enabled and text.active then

			text.active = false;
			text.value  = nil;				
			text:SetAlpha( 0 );
			text:SetText( "" );				
			
		elseif text.enabled then

			text.active = true;
			text:SetAlpha( 1 );
			
		end
		
		-- set text justification
		
		if text.justifyH ~= justifyH then
			text.justifyH = justifyH;
			text:SetJustifyH( justifyH );
		end
		
		if text.justigyV ~= justifyV then
			text.justifyV = justifyV;
			text:SetJustifyV( justifyV );
		end
	end
	
	if anchor then frame.applyAnchor( anchor ); end

--	nUI_ProfileStop();
	
end

-------------------------------------------------------------------------------

frame.applyAnchor = function( anchor )
	
--	nUI_ProfileStart( ProfileCounter, "applyAnchor" );
	
	local anchor      = anchor or frame.anchor or {};
	local anchor_pt   = anchor.anchor_pt or "CENTER";
    -- Dragonflight TODO
	local relative_to = anchor.relative_to or frame:GetParent():GetName();
	local relative_pt = anchor.relative_pt or anchor_pt;
	local xOfs        = (anchor.xOfs or 0) * nUI.hScale;
	local yOfs        = (anchor.yOfs or 0) * nUI.vScale;
	
	if frame.anchor_pt   ~= anchor_pt
	or frame.relative_to ~= relative_to
	or frame.relative_pt ~= relative_pt
	or frame.xOfs        ~= xOfs
	or frame.yOfs        ~= yOfs
	then
		
		frame.anchor_pt = anchor_pt;
		frame.relative_to = relative_to;
		frame.relative_pt = relative_pt;
		frame.xOfs        = xOfs;
		frame.yOfs        = yOfs;
		
		frame:ClearAllPoints();
		frame:SetPoint( anchor_pt, relative_to, relative_pt, xOfs, yOfs );
		
	end
	
	if frame.options and frame.text.enabled then
		
		local text  = frame.text;
		local label = frame.options.label;

		anchor_pt   = label.anchor_pt or "CENTER";
		relative_to = label.relative_to or frame:GetName();
		relative_pt = label.relative_pt or anchor_pt;
		xOfs        = (label.xOfs or 0) * frame.scale * nUI.hScale;
		yOfs        = (label.yOfs or 0) * frame.scale * nUI.vScale;
		
		if text.anchor_pt ~= anchor_pt
		or text.relative_to ~= relative_to
		or text.relative_pt ~= relative_pt
		or text.xOfs        ~= xOfs
		or text.yOfs        ~= yOfs
		then
			
			text.anchor_pt = anchor_pt;
			text.relative_to = relative_to;
			text.relative_pt = relative_pt;
			text.xOfs        = xOfs;
			text.yOfs        = yOfs;
			
			text:SetPoint( anchor_pt, relative_to, relative_pt, xOfs, yOfs );
			
		end
	end
	
--	nUI_ProfileStop();
	
end

-------------------------------------------------------------------------------

frame.applyOptions = function( options )
	
--	nUI_ProfileStart( ProfileCounter, "applyOptions" );
	
	frame.options = options;
	
	if not options or not options.enabled then

		frame.setEnabled( false );
		frame:Hide();
		
	else
		
		frame.setEnabled( true );
		frame:Show();
	
		background:SetFrameStrata( options.strata or background:GetParent():GetFrameStrata() );
		background:SetFrameLevel( background:GetFrameLevel() + (options.level or 1) );
		
		frame:SetFrameStrata( background:GetFrameStrata() );
		frame:SetFrameLevel( background:GetFrameLevel()+1 );
		
		-- set up the bar
		
		if not options.bar or not options.bar.enabled then
			
			frame.bar.enabled = false;
			frame.updateBar( nil, { 0, 0, 0, 0 } );
			
		else
			
			frame.bar.enabled = true;
			
			frame.setBar( nil, 0, 1 );
			frame.setOverlay( options.bar.overlay );
			frame.setOrientation( options.bar.orient );
			
		end
		
		-- if there's a border, set it
		
		if options.border then
				
			local border_color = options.border.color.border;
			local backdrop_color = options.border.color.backdrop;
			
            frame.Backdrop = CreateFrame("Frame", frame:GetName().."Backdrop", frame, "BackdropTemplate")
            frame.Backdrop:SetAllPoints()

			frame.Backdrop:SetBackdrop( options.border.backdrop );
			frame.Backdrop:SetBackdropBorderColor( border_color.r, border_color.g, border_color.b, border_color.a );
			frame.Backdrop:SetBackdropColor( backdrop_color.r, backdrop_color.g, backdrop_color.b, backdrop_color.a );
	
		else 
			
			--frame:SetBackdrop( nil );
			
		end
		
		-- if there's a background, set it
		
		if options.background then
				
			local border_color = options.background.color.border;
			local backdrop_color = options.background.color.backdrop;
			
            background.Backdrop = CreateFrame("Frame", background:GetName().."Backdrop", background, "BackdropTemplate")
            background.Backdrop:SetAllPoints()
            
            
			background:SetAlpha( 1 );
			background.Backdrop:SetBackdrop( options.background.backdrop );
			background.Backdrop:SetBackdropBorderColor( border_color.r, border_color.g, border_color.b, border_color.a );
			background.Backdrop:SetBackdropColor( backdrop_color.r, backdrop_color.g, backdrop_color.b, backdrop_color.a );
	
		else 
			
			background:SetAlpha( 0 );
			--background:SetBackdrop( nil );
			
		end
		
		frame.applyScale( options.scale or frame.scale or 1 );
		
	end
	
--	nUI_ProfileStop();
	
end

-------------------------------------------------------------------------------

frame.applySkin = function( skin )
	
--	nUI_ProfileStart( ProfileCounter, "applySkin" );
	
	local skin = skin and skin.AzeriteBar or nUI_DefaultConfig.AzeriteBar;
	
	if not skin or not skin.enabled then
		
		frame.setEnabled( false );
		frame:Hide();
		background:Hide();
		
	else
		
		frame.setEnabled( true );
		frame:Show();
		background:Show();
		
		frame.applyOptions( skin.options );
		frame.applyAnchor( skin.anchor );
		
	end
	
--	nUI_ProfileStop();
	
end
